#-*- coding:utf-8 -*-

ERROR_READ = 'Configuration file "%s", "%s" value is not %s'
ERROR_WRITE = '%s value is not ("%s", "%s")'

from System.IO import *
from System.Text import *

class InifileBase():
    """
        This class is Inheritance Base
    """
    def __init__(self, filename):
        """
            Attribute is the file name only
        """
        self.filename = filename

    def _get_value(self, section, key):
        """
            This Methods is Inheritance Base
        """
        return None

    def read_int(self, section, key, default):
        """
            @param default: int.
        """
        result = self._get_value(section, key)
        if result == None:
            return default
        try:
            return int(result)
        except:
            raise ValueError, ERROR_READ % (section, key, "int")
    
    def read_float(self, section, key, default):
        """
            @param default: float.
        """
        result = self._get_value(section, key)
        if result == None:
            return default
        try:
            return float(result)
        except:
            raise ValueError, ERROR_READ % (section, key, "float")
    
    def read_bool(self, section, key, default):
        """
            @param default: bool.
        """
        result = self._get_value(section, key)
        if result == None:
            return default
        if result == "1" or result == "-1":
            return True
        elif result == "0":
            return False
        else:
            raise ValueError, ERROR_READ % (section, key, "bool")
    
    def read_str(self, section, key, default):
        """
            @param default: str.
        """
        result = self._get_value(section, key)
        if result == None:
            return default
        return result


class InifileReader(InifileBase):
    """
        This class is read-only
        ini files to load faster
    """
    def __init__(self, filename):
        """
            @param filename: the full path name.
        """
        InifileBase.__init__(self, filename)

    def _get_value(self, section, key):
        """
            Read using the file stream
        """
        #if os.path.exists(self.filename):
        if File.Exists(self.filename):
            section_in = False
            #f = open(self.filename)
            f = StreamReader(self.filename, Encoding.UTF8)
            try:
                while 1:
                    line = f.ReadLine()
                    if line == None:
                        break
                    if line == "":
                        continue
                    if section_in:
                        if "=" in line:
                            pos = line.index("=")
                            if key == line[:pos]:
                                return line[pos+1:]
                        if len(line) > 2 and line[0] =="[" and line[-1] == "]":
                            return None
                    if len(line) > 2 and line[0] =="[" and line[-1] == "]":
                        if section == line[1:-1]:
                            section_in = True
            finally:
                f.Close()
        return None


class Inifile(InifileBase):
    """
        ini file to read and write class.
    """
    def __init__(self, filename):
        """
            Loading degradation inifile
            @param filename: the full path name.
        """
        InifileBase.__init__(self, filename)
        self.header = ""
        self.ini = []
        if File.Exists(self.filename):
            f = StreamReader(self.filename, Encoding.UTF8)
            try:
                section = ""
                while 1:
                    line = f.ReadLine()
                    if line == None:
                        break
                    if line == "":
                        continue
                    if len(line) > 2 and line[0] =="[" and line[-1] == "]":
                        section = line[1:-1]
                    elif section == "":
                        pass
                    elif "=" in line:
                        pos = line.index("=")
                        self._add(section, line[:pos], line[pos+1:])
            finally:
                f.Close()

    def _add(self, section, key, value):
        """
            Add to contents.
        """
        for dic1 in self.ini:
            if section in dic1.keys():
                for dic2 in dic1[section]:
                    if key in dic2.keys():
                        dic2[key] = value
                        return
                dic1[section].append({key: value})
                return
        self.ini.append({section: [{key: value}]})

    def _get_value(self, section, key):
        """
            Get to contents.
        """
        for dic1 in self.ini:
            if section in dic1.keys():
                for dic2 in dic1[section]:
                    if key in dic2.keys():
                        return dic2[key]
        return None
    
    def save(self):
        """
            Save the contents.
        """
        s = ""
        if self.header:
            s += self.header
        for dic1 in self.ini:
            for section in dic1.keys():
                s += "[" + section + "]" + "\n"
                for dic2 in dic1[section]:
                    for key in dic2.keys():
                        s += key + "=" + dic2[key] + "\n"
                s += "\n"
        if s != "":
            f = open(self.filename, "w")
            f.write(s)
            f.close()
    
    def section_exists(self, section):
        """
            Check existence of section.
        """
        for dic1 in self.ini:
            if section in dic1.keys():
                return True
        return False
    
    def erase_section(self, section):
        """
            Remove the specified section.
        """
        for dic1 in self.ini:
            if section in dic1.keys():
                dic1.pop(section)
                return True
        return False

    def erase_key(self, section, key):
        """
            EraseKey
        """
        for dic1 in self.ini:
            if section in dic1.keys():
                for dic2 in dic1[section]:
                    if key in dic2.keys():
                        dic2.pop(key)
                        return True
        return False

    def write_int(self, section, key, value):
        """
            @param value: int.
        """
        if isinstance(value, int):
            self._add(section, key, str(value))
        else:
            raise ValueError, ERROR_WRITE % ("int", section, key)
    
    def write_float(self, section, key, value):
        """
            @param value: float.
        """
        if isinstance(value, float):
            self._add(section, key, str(value))
        else:
            raise ValueError, ERROR_WRITE % ("froat", section, key)
    
    def write_bool(self, section, key, value):
        """
            @param value: bool.
        """
        if isinstance(value, bool):
            if value:
                self._add(section, key, "1")
            else:
                self._add(section, key, "0")
        else:
            raise ValueError, ERROR_WRITE % ("bool", section, key)
    
    def write_str(self, section, key, value):
        """
            @param value: str.
        """
        if isinstance(value, str):
            self._add(section, key, value)
        else:
            raise ValueError, ERROR_WRITE % ("str", section, key)

